# tetra
--
    import "github.com/Xe/Tetra/bot"


## Usage

#### type ChanUser

```go
type ChanUser struct {
	Client  *Client  `json:"client"`
	Channel *Channel `json:"channel"`
	Prefix  int
}
```


#### type Channel

```go
type Channel struct {
	Name    string               `json:"name"`
	Ts      int64                `json:"ts"`
	Modes   int                  `json:"modes"`
	Clients map[string]*ChanUser `json:"clients"`
	Lists   map[int][]string
	Gauge   metrics.Gauge
}
```

Implements Targeter

#### func (*Channel) AddChanUser

```go
func (c *Channel) AddChanUser(client *Client) (cu *ChanUser)
```

#### func (*Channel) DelChanUser

```go
func (c *Channel) DelChanUser(client *Client) (err error)
```

#### func (*Channel) IsChannel

```go
func (c *Channel) IsChannel() bool
```

#### func (*Channel) Target

```go
func (c *Channel) Target() string
```

#### type Client

```go
type Client struct {
	Nick        string
	User        string
	Host        string
	VHost       string
	Ip          string
	Account     string
	Uid         string
	Gecos       string
	Permissions int
	Umodes      int
	Kind        string

	Ts       int64
	Channels map[string]*Channel
	Server   *Server
	Commands map[string]*Command
}
```


#### func (*Client) Euid

```go
func (r *Client) Euid() string
```

#### func (*Client) IsChannel

```go
func (r *Client) IsChannel() bool
```

#### func (*Client) IsOper

```go
func (r *Client) IsOper() bool
```

#### func (*Client) Join

```go
func (r *Client) Join(channame string)
```

#### func (*Client) Notice

```go
func (r *Client) Notice(destination Targeter, message string)
```

#### func (*Client) Part

```go
func (r *Client) Part(channame string) bool
```

#### func (*Client) Privmsg

```go
func (r *Client) Privmsg(destination Targeter, message string)
```

#### func (*Client) Quit

```go
func (r *Client) Quit()
```

#### func (*Client) ServicesLog

```go
func (r *Client) ServicesLog(message string)
```

#### func (*Client) Target

```go
func (r *Client) Target() string
```

#### type Clients

```go
type Clients struct {
	ByNick map[string]*Client
	ByUID  map[string]*Client
	Gauge  metrics.Gauge
	Tetra  *Tetra
}
```


#### func (*Clients) AddClient

```go
func (clients *Clients) AddClient(client *Client)
```

#### func (*Clients) DelClient

```go
func (clients *Clients) DelClient(client *Client) (err error)
```

#### type Command

```go
type Command struct {
	Impl   func(*Client, Targeter, []string) string
	Uuid   string
	Script *Script
	Verb   string
	Owner  *Client
}
```

Struct command holds everything needed for a bot command.

#### func  NewCommand

```go
func NewCommand(c *Client, verb string, handler func(*Client, Targeter, []string) string) (cmd *Command, err error)
```
NewCommand returns a new command instance.

#### type Config

```go
type Config struct {
	Autoload []string
	Services []*ServiceConfig
	Server   *ServerConfig
	Uplink   *UplinkConfig
	Stats    *StatsConfig
	ApiKeys  map[string]string
}
```


#### func  NewConfig

```go
func NewConfig(fname string) (conf *Config, err error)
```

#### type Connection

```go
type Connection struct {
	Conn   net.Conn
	Log    *log.Logger
	Reader *bufio.Reader
	Tp     *textproto.Reader
	Buffer chan string
}
```


#### func (*Connection) Close

```go
func (c *Connection) Close()
```

#### func (*Connection) GetLine

```go
func (c *Connection) GetLine() (line string, err error)
```

#### func (*Connection) SendLine

```go
func (c *Connection) SendLine(line string, stuff ...interface{})
```

#### type Handler

```go
type Handler struct {
	Impl   func(*r1459.RawLine)
	Verb   string
	Uuid   string
	Script *Script
	Go     bool
}
```


#### type Script

```go
type Script struct {
	Name     string
	L        *lua.State
	Tetra    *Tetra
	Log      *log.Logger
	Handlers map[string]*Handler
	Commands map[string]*Command
	Service  string
	Client   *Client
	Uuid     string
}
```


#### func (*Script) AddLuaCommand

```go
func (script *Script) AddLuaCommand(verb string, name string) error
```
AddLuaCommand adds a new command to a script from a lua context

#### func (*Script) AddLuaProtohook

```go
func (script *Script) AddLuaProtohook(verb string, name string) error
```
AddLuaProtohook adds a lua function as a protocol hook

#### type Server

```go
type Server struct {
	Sid   string    `json:"sid"`
	Name  string    `json:"name"`
	Gecos string    `json:"gecos"`
	Links []*Server `json:"links"`

	Counter metrics.Gauge
}
```


#### func (*Server) AddClient

```go
func (s *Server) AddClient()
```

#### func (*Server) DelClient

```go
func (s *Server) DelClient()
```

#### type ServerConfig

```go
type ServerConfig struct {
	Name      string
	Gecos     string
	Sid       string
	StaffChan string
	SnoopChan string
}
```


#### type ServiceConfig

```go
type ServiceConfig struct {
	Nick  string
	User  string
	Host  string
	Gecos string
	Name  string
}
```


#### type StatsConfig

```go
type StatsConfig struct {
	Host     string
	Database string
	Username string
	Password string
}
```


#### type Targeter

```go
type Targeter interface {
	Target() string
	IsChannel() bool
}
```


#### type Tetra

```go
type Tetra struct {
	Conn     *Connection
	Info     *Server
	Clients  *Clients
	Channels map[string]*Channel
	Bursted  bool
	Handlers map[string]map[string]*Handler
	Services map[string]*Client
	Servers  map[string]*Server
	Scripts  map[string]*Script

	Config *Config
	Log    *log.Logger
	Uplink *Server
}
```


#### func  NewTetra

```go
func NewTetra(cpath string) (tetra *Tetra)
```

#### func (*Tetra) AddHandler

```go
func (tetra *Tetra) AddHandler(verb string, impl func(*r1459.RawLine)) (handler *Handler, err error)
```

#### func (*Tetra) AddService

```go
func (tetra *Tetra) AddService(service, nick, user, host, gecos string) (cli *Client)
```

#### func (*Tetra) Auth

```go
func (tetra *Tetra) Auth()
```

#### func (*Tetra) Burst

```go
func (tetra *Tetra) Burst()
```

#### func (*Tetra) Connect

```go
func (tetra *Tetra) Connect(host, port string) (err error)
```

#### func (*Tetra) DelHandler

```go
func (tetra *Tetra) DelHandler(verb string, uuid string) (err error)
```

#### func (*Tetra) DelService

```go
func (tetra *Tetra) DelService(service string) (err error)
```

#### func (*Tetra) GetChannelStats

```go
func (t *Tetra) GetChannelStats()
```

#### func (*Tetra) GetConn

```go
func (tetra *Tetra) GetConn() *net.Conn
```

#### func (*Tetra) GetNetworkStats

```go
func (t *Tetra) GetNetworkStats()
```
Continuously reports the network statistics. Should be run in a gorotutine.

#### func (*Tetra) LoadScript

```go
func (tetra *Tetra) LoadScript(name string) (script *Script, err error)
```

#### func (*Tetra) Main

```go
func (t *Tetra) Main()
```

#### func (*Tetra) NewChannel

```go
func (tetra *Tetra) NewChannel(name string, ts int64) (c *Channel)
```

#### func (*Tetra) NextUID

```go
func (tetra *Tetra) NextUID() string
```

#### func (*Tetra) ProcessLine

```go
func (tetra *Tetra) ProcessLine(line string)
```

#### func (*Tetra) Quit

```go
func (tetra *Tetra) Quit()
```

#### func (*Tetra) StickConfig

```go
func (tetra *Tetra) StickConfig()
```

#### func (*Tetra) UnloadScript

```go
func (tetra *Tetra) UnloadScript(name string) error
```
Unload a script and delete its commands and handlers

#### func (*Tetra) WebApp

```go
func (t *Tetra) WebApp()
```

#### type UplinkConfig

```go
type UplinkConfig struct {
	Host     string
	Port     string
	Password string
	Ssl      bool
}
```
